% Bin_14ACC_Matlab
%
% This file reconstructs the results in Section V of:
%
% Worst-Case False Alarm Analysis of Fault Detection Systems
% American Control Conference, 2014
% 
% The paper describes how to embed the model uncertainty into false alarm
% analysis and compute worst-case false alarm probability over a certain
% time window.
% The aim of the example is showing worst-case second order sidak's bound
% can be less conservative than the worst-case first order bound.
% The computation in this file relies on chebfun toolbox. Please download
% the chebfun toolbox before running this file. The bounds computed here
% also make use of the matlab function "mvncdf". This function is
% implemented in the Matlab statistical toolbox and is used to compute two
% dimensional and three dimensional Gaussian integrals.
%


%%clear;
%construct the frequency w as a cheb function
w = chebfun('w',[0 pi]);
z=exp(1i*w);
%The related block diagrams are Figure 1 and Figure 2 in the ACC paper
%Propotional Control gain K for the controller in Figure 2
K=-5;
%The aircraft dynamics G0 in Figure 1
%This is a discrete time version of the lateralCdirectional reduced
%order model descrbied in the Example 7.2 in the following book:
%M. V. Cook. Flight Dynamics Principles, Second Edition: A Linear
%Systems Approach to Aircraft Stability and Control. Butterworth-
%Heinemann, 2nd edition, 2007.
G0=-0.0161./(z-0.9878);
%A first order model for wind gust dynamics w(k) in Figure 1 and Figure 2
%in the ACC paper
W=0.5./(z-0.995);
%In Figure 2, the transfer function from y_k to r_k is Part1, this transfer
%function can be used to construct the transfer function from noise v_k to
%residuals r_k
Part1=1+K.*G0;
%A stable minimum phase transfer function Wu as described in Equation 1 of
%the ACC paper
Wu=0.1;
% The system is operational at 100Hz and one hour corresponds to N=360000
% samples. We want to compuate false alarm probability per hour and hence
% we are going to compute the false alarm probability over a discrete time
% window of length N=360000.
N=360000;
%The threshold for the fault detection decision logic is chosen at T=45.
T=45;
l=1;
%set initial values for spectrum
%delta is a gain uncertainty
for delta=-1:0.1:-1
    %In Figure 1, the transfer function of the entire aircraft dynamic
    %system is G_ac
    G_ac=(1+Wu*delta).*G0;
    %In Figure 2, the transfer function from noise v_k to y_k is Part2,
    %this transfer function can be used to construct the transfer function
    %from noise v_k to residual r_k and also the transfer function from the
    %innovation of the disturbance to the residual r_k
    Part2=1+G_ac.*K;
    %spectrum of the FDI residuals is Phi_R, in includes two parts, one is
    %generated from the noise v_k and the other is generated by the wind
    %gust. Here 1.3 is the variance of the innovation of the wind gust.
    Phi_R=abs(Part1./Part2).^2+abs((G_ac.*Part1.*W)./Part2).^2*1.3;
    %infimum of the spectrum Phi_R will be stored in phi1
    phi1=Phi_R;
    %supremum of the spectrum Phi_R will be stored in phi2
    phi2=Phi_R;    
end
% For each delta, compute bounds (1-gamma1) and (1-gamma2). These two
% quantities are upper bounds for false alarm probability per hour.
% (1-gamma1) is the so-called sidak's bound and (1-gamma2) is the second
% order sidak's bound. (1-gamma1) involves computation of 1 dimensional
% Gaussian integral and (1-gamma2) involves computation of 2 dimensional
% Gaussian integral.
for delta=-1:0.1:1
    %For each delta,the transfer function of the entire aircraft dynamic
    %system G_ac is computed as(1+Wu*delta)*G0
    G_ac=(1+Wu*delta).*G0;
    %For each delta, the transfer function from v_k to y_k is denoted as
    %Part2 and computed by (1+G_ac*K)
    Part2=1+G_ac.*K;
    Phi_R=abs(Part1./Part2).^2+abs((G_ac.*Part1.*W)./Part2).^2*1.3;
    %update the infimum of the spectrum of residual
    phi1=min(Phi_R,phi1);
    %update the superemum of the spectrum of residual
    phi2=max(Phi_R,phi2);
    %update beta0 (variance) and beta1 (first order covariance) based on
    %Equation 6 in the ACC paper. beta0 is needed for both computation of
    %(1-gamma1) and (1-gamma2). beta1 is needed to compute(1-gamma2).
    beta0=sum(abs(Part1./Part2).^2+abs((G_ac.*Part1.*W)./Part2).^2*1.3)/pi;
    beta1=sum(cos(w).*(abs(Part1./Part2).^2+abs((G_ac.*Part1.*W)./Part2).^2*1.3))/pi;
    %compute the Q2(delta) from the matlab function mvncdf
    Q2=mvncdf([-T -T],[T T],[0 0],[beta0 beta1;beta1 beta0]);
    %compute the Q1(delta)
    Q1=mvncdf(-T,T,0,beta0);
    %compute (1-gamma1) for this delta
    OneminusGamma1(l)=1-Q1^N;
    %compute (1-gamma2) for this delta
    OneminusGamma2(l)=1-(Q2/Q1)^(N-2)*Q2;
    l=l+1;
end


%Use the method in Section 4.2 to do the worst case analysis. Directly
%searching for the worst case bound alpha2 based on a fine grid of c
l=1;
%grid c between [0 pi]
for c=0:0.1:pi
    %apply the operator Mminus described by Equations 28 and 29 in ACC paper
beta0=sum(phi1,[0 c])/pi+sum(phi2,[c pi])/pi;
beta1=sum(phi1.*cos(w),[0 c])/pi+sum(phi2.*cos(w),[c pi])/pi;
%compute (1-gamma2) for this c value
Q2=mvncdf([-T -T],[T T],[0 0],[beta0 beta1;beta1 beta0]);
    Q1=mvncdf(-T,T,0,beta0);
    %store the candidate maximum
    Gamma3(l)=1-(Q2/Q1)^(N-2)*Q2;
    l=l+1;
    %apply the operator Mplus
    beta0=sum(phi2,[0 c])/pi+sum(phi1,[c pi])/pi;
beta1=sum(phi2.*cos(w),[0 c])/pi+sum(phi1.*cos(w),[c pi])/pi;
Q2=mvncdf([-T -T],[T T],[0 0],[beta0 beta1;beta1 beta0]);
    Q1=mvncdf(-T,T,0,beta0);
    %store the candidate maximum
    Gamma3(l)=1-(Q2/Q1)^(N-2)*Q2;
    l=l+1;
end
%compute the worst case second order bound alpha2
alpha2=max(Gamma3(:));
%Plot the results
plot(-1:0.1:1,OneminusGamma1,'r--',-1:0.1:1,OneminusGamma2,'b.-.',-1:0.1:1,(-1:0.1:1).*0+alpha2,'k');
xlabel('\Delta');
ylabel('Probability Bounds');
legend('1-\gamma_N^{(1)}','1-\gamma_N^{(2)}','\alpha_N^{(2)}');
grid on;
set(gca,'Xtick',-1:0.5:1);
